//---------------------------------------------------------------------
// Copyright By vtron  Corporation.
// vtron Confidential Proprietary
//---------------------------------------------------------------------
// File name: syn_code_16.v
// Module name: syn_code_16
// Module Description: used to synchronize the video with system vs signal
// Created by: caojie
// ---------------------------------------------------------------------
// Release history
// VERSION |   Date      | AUTHOR  |    DESCRIPTION
//   1.0   |  7-Sep-2009 | caojie  |      initial
//   1.1   | 26-Jan-2011 | caojie  |      dra_325
//   1.2   |  3-Mar-2011 | caojie  |      hd_syn_code
//   1.3   | 14-Dec-2011 | caojie  |      sd_syn_code
//   1.4   | 04-Apr-2015 | caojie  |  ޸ģ,ͨΪ3
//   1.5   |04-Jul-2015  | caojie  |  ͨΪ1
// ---------------------------------------------------------------------


module syn_code_16(
    // inputs
    input  [15:0] cpu2out_xst_reg   ,
    input  [15:0] cpu2out_xed_reg   ,
    input  [15:0] cpu2out_yst_reg   ,
    input  [15:0] cpu2out_yed_reg   ,
    input  [15:0] cpu2out_hsync_reg ,
    input  [15:0] cpu2out_vsync_reg ,
    input  [15:0] cpu2out_hed_reg   ,
    input  [15:0] cpu2out_ved_reg   ,
                  
    input         vs_sel            ,
    input         rst_n             ,
    input         clk_81m           ,
    input         e_vs              ,
    input  [2:0]  blue_out          ,
                  
    input  [15:0] off0_syn_data     ,
                  
    input  [15:0] hor_res0          ,
    input  [15:0] ver_res0          ,
    
    // outputs
    output        syn_off0_re       ,
    output        syn_off0_vs       ,
    output        syn_off0_hs       ,
    output        syn_clip0_vs      ,
    output        syn_clip0_hs      ,
    output        syn_clip0_de      ,
    output [15:0] syn_clip0_data     
    );

//==========================================
  // wires
  wire   pout_hs;
  wire   pout_vs;
  
  wire   off0_re;
  
  // regs  
  reg   off0_re_d1;
  reg   off0_re_d2;

  wire  [15:0] blueout_data;

//============================================================================
  assign syn_off0_re   = off0_re;
  assign syn_off0_vs   = pout_vs;
  assign syn_off0_hs   = pout_hs;

//==================================================================
blueout_ycbcr blueout_ycbcr_inst(
    .rst_ni  (rst_n       ),
    .clk_i   (clk_81m     ),
    .hs_i    (pout_hs     ),
    .ycbcr_o (blueout_data)
    );

//============================================================================
  assign syn_clip0_de   = off0_re_d1;
  assign syn_clip0_hs   = pout_hs ; 
  assign syn_clip0_vs   = pout_vs ; 
  assign syn_clip0_data = blue_out[0] ? blueout_data: off0_syn_data;

//===================== system signal generate =====================
  sync_gen1x sync_gen1x_inst(
     //-------
     .cpu2out_xst_reg   (cpu2out_xst_reg     ),
     .cpu2out_xed_reg   (cpu2out_xed_reg     ), 
     .cpu2out_yst_reg   (cpu2out_yst_reg     ), 
     .cpu2out_yed_reg   (cpu2out_yed_reg     ),
     .cpu2out_fxed_reg  (16'd0),  
     .cpu2out_fyed_reg  (16'd0), 
     .cpu2out_hsync_reg (cpu2out_hsync_reg   ) ,
     .cpu2out_vsync_reg (cpu2out_vsync_reg   ) , 
     .cpu2out_hed_reg   (cpu2out_hed_reg     ) ,
     .cpu2out_ved_reg   (cpu2out_ved_reg     ) ,
       
     .hor_res0          (hor_res0            ),
     .ver_res0          (ver_res0            ),
     //-------  video   
     .off0_re           (off0_re             ),
             
     .pout_de           (                    ),//unused 
     .pout_hs           (pout_hs             ),
     .pout_vs           (pout_vs             ), 
     .vs_sel            (vs_sel              ),                            
     .e_vs              (e_vs                ),         
     .pxl_clk           (clk_81m             ),
     .rst_b             (rst_n               )
     );

//==================== delay ofifo read enable signal ====================
  always@(posedge clk_81m or negedge rst_n)
  begin
    if(rst_n == 1'b0) 
      begin
        off0_re_d1 <= 1'b0;
        off0_re_d2 <= 1'b0;
      end
    else 
      begin
        off0_re_d1 <= off0_re;
        off0_re_d2 <= off0_re_d1;
      end
  end

endmodule

