//---------------------------------------------------------------------
// File name  : tb.v
// Module name: tb
// Created by : 
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date       | AUTHOR     |    DESCRIPTION
// ---------------------------------------------------------------------
//  1.0    | 15-Nov-2011 |            |     initial 
// ---------------------------------------------------------------------

`timescale 1ns / 1ps  

//According to IP parameters to choose
`define	WR_VIDEO_WIDTH_16
`define	DEF_WR_VIDEO_WIDTH 16
//`define	WR_VIDEO_WIDTH_24
//`define	DEF_WR_VIDEO_WIDTH 24
//`define	WR_VIDEO_WIDTH_32
//`define	DEF_WR_VIDEO_WIDTH 32

`define	RD_VIDEO_WIDTH_16
`define	DEF_RD_VIDEO_WIDTH 16
//`define	RD_VIDEO_WIDTH_24
//`define	DEF_RD_VIDEO_WIDTH 24
//`define	RD_VIDEO_WIDTH_32
//`define	DEF_RD_VIDEO_WIDTH 32

`define	USE_THREE_FRAME_BUFFER

`define	DEF_ADDR_WIDTH 21 
`define	DEF_SRAM_DATA_WIDTH 64

module tb();
//========================================================
//parameters
parameter BMP0_VIDEO_FORMAT         = "WxH_xHz"; //video format
parameter BMP0_PIXEL_CLK_FREQ       = 50.000;//pixel clock frequency, unit: MHz
parameter BMP0_PIXEL_CLK_PERIOD     = 1000.0/BMP0_PIXEL_CLK_FREQ; //unit: ns
parameter BMP0_WIDTH                = 80;
parameter BMP0_HEIGHT               = 60;
parameter BMP0_OPENED_NAME          = "../../tb/pic/img80.bmp";
parameter BMP0_REPEAT               = 1'b1;  //0:bmp increase  , 1:bmp repeat 
parameter BMP0_LINK				    = 1'b0;  //0:single pixel  , 1:dual pixel  
                                    
parameter BMP0_OUT_VIDEO_FORMAT     = "WxH_xHz"; //video format
parameter BMP0_OUT_PIXEL_CLK_FREQ   = 50.000;//pixel clock frequency, unit: MHz
parameter BMP0_OUT_PIXEL_CLK_PERIOD = 1000.0/BMP0_OUT_PIXEL_CLK_FREQ; //unit: ns
parameter BMP0_OUTPUTED_WIDTH       = BMP0_WIDTH;
parameter BMP0_OUTPUTED_HEIGHT      = BMP0_HEIGHT;
parameter BMP0_OUTPUTED_NAME        = "../../tb/pic/out0_001.bmp";
parameter BMP0_OUTPUTED_NUMBER      = 16'd3;

//=========================================================
//SRAM parameters
parameter ADDR_WIDTH          = `DEF_ADDR_WIDTH        ;   //Total capacity =2^21*32bit = 64Mbit 
parameter DATA_WIDTH          = `DEF_SRAM_DATA_WIDTH   ;   //psram 64Mbit, x32 user date width 64bit
parameter WR_VIDEO_WIDTH	  = `DEF_WR_VIDEO_WIDTH    ;  
parameter RD_VIDEO_WIDTH	  = `DEF_RD_VIDEO_WIDTH;  
//-------------------------------------------------
parameter DDR_REF_CLK_FREQ   = 27.000;//ddr reference clock frequency, unit: MHz
parameter DDR_REF_CLK_PERIOD = 1000.0/DDR_REF_CLK_FREQ; //unit: ns 

//=======================================================
reg  ddr_ref_clk;      // 50M
reg  rst_n      ;

reg  output_pixel_clk;

reg  rd_gen ;

//----------------  
//config
wire  [15:0] data_valid_xst_o;
wire  [15:0] data_valid_xed_o;
wire  [15:0] data_valid_yst_o;
wire  [15:0] data_valid_yed_o;
wire  [15:0] hor_sync_time_o ;
wire  [15:0] ver_sync_time_o ;
wire  [15:0] hor_total_time_o;
wire  [15:0] ver_total_time_o;

//psram
wire        dma_clk  ; 

wire        memory_clk;
wire        mem_pll_lock  ;

//-------------------------------------------------
//memory interface
wire [1:0]  O_psram_ck;
wire [1:0]  O_psram_ck_n;
wire [1:0]  IO_psram_rwds;
wire [15:0] IO_psram_dq;
wire [1:0]  O_psram_reset_n;
wire [1:0]  O_psram_cs_n;

wire                    cmd           ;
wire                    cmd_en        ;
wire [ADDR_WIDTH-1:0]   addr          ;//[ADDR_WIDTH-1:0]
wire [DATA_WIDTH-1:0]   wr_data       ;//[DATA_WIDTH-1:0]
wire [DATA_WIDTH/8-1:0] data_mask     ;
wire                    rd_data_valid ;
wire [DATA_WIDTH-1:0]   rd_data       ;//[DATA_WIDTH-1:0]
wire                    init_calib     ;

//-------------------------------------------------------------------------------
reg  ch0_pixel_clock; 

//------------
//dirver
wire       ch0_vsync; 
wire       ch0_hsync; 
wire       ch0_data_valid; 
wire [7:0] ch0_data0_r; 
wire [7:0] ch0_data0_g;
wire [7:0] ch0_data0_b;
wire [7:0] ch0_data1_r; 
wire [7:0] ch0_data1_g;
wire [7:0] ch0_data1_b;

//-----------------------
wire        ch0_clk_in ; 
wire        ch0_vs_in  ;
wire        ch0_hs_in  ;
wire        ch0_de_in  ;
wire [23:0] ch0_data_in;  
  
//-------------------------
//frame buffer in
wire                      ch0_vfb_vs_in  ;
wire                      ch0_vfb_de_in  ;
wire [WR_VIDEO_WIDTH-1:0] ch0_vfb_data_in;

//-------------------
//syn_code
wire                      syn_off0_re;  // ofifo read enable signal
wire                      syn_off0_vs;
wire                      syn_off0_hs;
                          
wire                      off0_syn_de  ;
wire [RD_VIDEO_WIDTH-1:0] off0_syn_data;

//-------------------------
//yc422 to rgb
wire        ch0_vs_rgb_o  ;
wire        ch0_hs_rgb_o  ;
wire        ch0_de_rgb_o  ;
wire [7:0]  ch0_data_r_o  ;
wire [7:0]  ch0_data_g_o  ;
wire [7:0]  ch0_data_b_o  ;

//-----------------
//monitor rgb input
wire        m0_clk    ;
wire        m0_vs_rgb ;  
wire        m0_hs_rgb ;  
wire        m0_de_rgb ;  
wire [7:0]  m0_data0_r;
wire [7:0]  m0_data0_g;
wire [7:0]  m0_data0_b;
wire [7:0]  m0_data1_r;
wire [7:0]  m0_data1_g;
wire [7:0]  m0_data1_b;


//========================================================
GSR GSR(.GSRI(1'b1));

//==============================================  
initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end


//==============================================  
//rst_n
initial
  begin
    rst_n=1'b0;
    
    #2000;
    rst_n=1'b1;
end

//=====================================================
//pixel_clk
initial
begin
	ch0_pixel_clock = 1'b0;
	output_pixel_clk = 1'b0;
end

always  #(BMP0_PIXEL_CLK_PERIOD/2.0) ch0_pixel_clock = ~ch0_pixel_clock;
always  #(BMP0_OUT_PIXEL_CLK_PERIOD/2.0) output_pixel_clk = ~output_pixel_clk;

//----------------------------------------------------
//ddr ref clk
initial 
begin
	ddr_ref_clk = 1'b0;
end

always  #(DDR_REF_CLK_PERIOD/2.0) ddr_ref_clk = ~ddr_ref_clk;

//==================================================
//video driver
driver #
(
	.BMP_VIDEO_FORMAT    (BMP0_VIDEO_FORMAT  ),
	.BMP_PIXEL_CLK_FREQ  (BMP0_PIXEL_CLK_FREQ),
	.BMP_WIDTH           (BMP0_WIDTH         ),
	.BMP_HEIGHT          (BMP0_HEIGHT        ),
	.BMP_OPENED_NAME     (BMP0_OPENED_NAME   )
)
driver_inst0
(
	.link_i      (BMP0_LINK       ), 
	.repeat_en   (BMP0_REPEAT     ),
	.video_gen_en(init_calib       ),//rst_n           ),
	.pixel_clock (ch0_pixel_clock ),
	.vsync       (ch0_vsync       ), 
	.hsync       (ch0_hsync       ), 
	.data_valid  (ch0_data_valid  ), 
	.data0_r     (ch0_data0_r     ), 
	.data0_g     (ch0_data0_g     ),
	.data0_b     (ch0_data0_b     ),
	.data1_r     (ch0_data1_r     ), 
	.data1_g     (ch0_data1_g     ),
	.data1_b     (ch0_data1_b     )
);

//==============================================
//dut
//----------------------------
assign ch0_clk_in   = ch0_pixel_clock; 
assign ch0_vs_in    = ch0_vsync;
assign ch0_hs_in    = ch0_hsync;
assign ch0_de_in    = ch0_data_valid;
assign ch0_data_in  = {ch0_data0_b,ch0_data0_g,ch0_data0_r};

//==============================================
`ifdef WR_VIDEO_WIDTH_16        
	assign ch0_vfb_vs_in   = ch0_vs_in;  
	assign ch0_vfb_de_in   = ch0_de_in;  
	assign ch0_vfb_data_in = {ch0_data_in[23:19],ch0_data_in[15:10],ch0_data_in[7:3]}; // RGB565
`endif

`ifdef WR_VIDEO_WIDTH_24
	assign ch0_vfb_vs_in   = ch0_vs_in;
	assign ch0_vfb_de_in   = ch0_de_in;
	assign ch0_vfb_data_in = ch0_data_in;
`endif 

`ifdef WR_VIDEO_WIDTH_32
	assign ch0_vfb_vs_in   = ch0_vs_in;
	assign ch0_vfb_de_in   = ch0_de_in;
	assign ch0_vfb_data_in = {8'd0,ch0_data_in};
`endif  
    
//=====================================================
VFB_PSRAM_Top VFB_PSRAM_Top_inst
( 
	.I_rst_n		    (init_calib       ),//rst_n            ),
	.I_dma_clk		    (dma_clk          ),   //sram_clk         ),
`ifdef USE_THREE_FRAME_BUFFER
	.I_wr_halt		    (1'd0             ), //1:halt,  0:no halt
	.I_rd_halt		    (1'd0             ), //1:halt,  0:no halt
`endif
	// video data input           
	.I_vin0_clk		    (ch0_clk_in       ),
	.I_vin0_vs_n	    (ch0_vfb_vs_in    ),
	.I_vin0_de		    (ch0_vfb_de_in    ),
	.I_vin0_data	    (ch0_vfb_data_in  ),
	.O_vin0_fifo_full   (                 ),
	// video data output          
	.I_vout0_clk	    (output_pixel_clk ),
	.I_vout0_vs_n	    (syn_off0_vs      ),
	.I_vout0_de		    (syn_off0_re      ),
	.O_vout0_den	    (off0_syn_de      ),
	.O_vout0_data	    (off0_syn_data    ),
	.O_vout0_fifo_empty (                 ),
	// ddr write request
	.O_cmd              (cmd              ),
	.O_cmd_en           (cmd_en           ),
	.O_addr             (addr             ),//[ADDR_WIDTH-1:0]
	.O_wr_data          (wr_data          ),//[DATA_WIDTH-1:0]
	.O_data_mask        (data_mask        ),
	.I_rd_data_valid    (rd_data_valid    ),
	.I_rd_data          (rd_data          ),//[DATA_WIDTH-1:0]
	.I_init_calib       (init_calib       )
);  

//----------------------------------------------------- 
//psram ip
gowin_rpll gowin_rpll_inst
(
    .clkout(memory_clk    ), //output clkout
    .lock  (mem_pll_lock  ), //output lock
    .clkin (ddr_ref_clk   )  //input clkin
);

PSRAM_Memory_Interface_HS_Top PSRAM_Memory_Interface_HS_Top_inst
(
	.clk            (ddr_ref_clk    ),
	.memory_clk     (memory_clk     ),
	.pll_lock       (mem_pll_lock   ),
	.rst_n          (rst_n          ),  //rst_n
	.O_psram_ck     (O_psram_ck     ),
	.O_psram_ck_n   (O_psram_ck_n   ),
	.IO_psram_rwds  (IO_psram_rwds  ),
	.IO_psram_dq    (IO_psram_dq    ),
	.O_psram_reset_n(O_psram_reset_n),
	.O_psram_cs_n   (O_psram_cs_n   ),
	.wr_data        (wr_data        ),
	.rd_data        (rd_data        ),
	.rd_data_valid  (rd_data_valid  ),
	.addr           (addr           ),
	.cmd            (cmd            ),
	.cmd_en         (cmd_en         ),
	.clk_out        (dma_clk        ),
	.data_mask      (data_mask      ),
	.init_calib     (init_calib     )
);
  
//-------------------------------------------------------------
//PSRAM model
wire VCC;
assign VCC = 1'b1;
W955D8MKY psram_mode0
(
	.resetb           (O_psram_reset_n[0]),
	.clk              (O_psram_ck[0]     ),
	.clk_n            (O_psram_ck_n[0]   ),
	.ceb              (O_psram_cs_n[0]   ),
	.adq              (IO_psram_dq[7:0]  ),
	.rwds             (IO_psram_rwds[0]  ),
	.VCC              (VCC),
	.VSS              (1'b0) 
);
      
W955D8MKY psram_model
(
	.resetb           (O_psram_reset_n[1]),
	.clk              (O_psram_ck_n[1]   ),//Attention, clock +/- exchange, only for simulation
	.clk_n            (O_psram_ck[1]     ),//
	.ceb              (O_psram_cs_n[1]   ),
	.adq              (IO_psram_dq[15:8] ),
	.rwds             (IO_psram_rwds[1]  ),
	.VCC              (VCC),
	.VSS              (1'b0) 
);

initial
begin
force tb.O_psram_cs_n[1:0] = 2'b11;
//#127150
#63575;
//#79468;
release tb.O_psram_cs_n;
end

initial
begin
force tb.VCC = 1'b0;
#803500 //#1203500
//#502187; //#401750;
//#502187;
release tb.VCC;
end  

//==============================================
//begin to read data after delay some time 
initial 
begin
	rd_gen = 1'b0 ;
	forever
	begin
		#100; //ns
		if ( init_calib == 1'b1 ) //init_calib //rst_n
			begin
				#600000;//ns
				rd_gen=1'b1;
			end 
		else 
			begin
				rd_gen=1'b0;
			end 
	end 
end 

//-----------------------------------------------------
//config
config_m#
(
	.OUTPUT_VIDEO_FORMAT   (BMP0_OUT_VIDEO_FORMAT  ),
	.OUTPUT_PIXEL_CLK_FREQ (BMP0_OUT_PIXEL_CLK_FREQ),
	.OUTPUT_HOR_RESOLUTION (BMP0_OUTPUTED_WIDTH    ),
	.OUTPUT_VER_RESOLUTION (BMP0_OUTPUTED_HEIGHT   )
)
config_m_inst
(
	.data_valid_xst_o (data_valid_xst_o), 
	.data_valid_xed_o (data_valid_xed_o), 
	.data_valid_yst_o (data_valid_yst_o), 
	.data_valid_yed_o (data_valid_yed_o),
	.hor_sync_time_o  (hor_sync_time_o ),
	.ver_sync_time_o  (ver_sync_time_o ),
	.hor_total_time_o (hor_total_time_o), 
	.ver_total_time_o (ver_total_time_o)  
);

//---------------------------------------------------
//generate synchronous timing
syn_code_16 syn_code_16_inst
(
	.cpu2out_xst_reg   (data_valid_xst_o),
	.cpu2out_xed_reg   (data_valid_xed_o),
	.cpu2out_yst_reg   (data_valid_yst_o),
	.cpu2out_yed_reg   (data_valid_yed_o),
	.cpu2out_hsync_reg (hor_sync_time_o ),
	.cpu2out_vsync_reg (ver_sync_time_o ),
	.cpu2out_hed_reg   (hor_total_time_o),
	.cpu2out_ved_reg   (ver_total_time_o),
	.vs_sel            (1'b0            ),//sim only
	.rst_n             (rd_gen          ),
	.clk_81m           (output_pixel_clk),
	.e_vs              (1'b0            ),//sim only
	.blue_out          (3'b000          ),
	.off0_syn_data     (                ),
	.hor_res0          (data_valid_xed_o-data_valid_xst_o+1'b1),
	.ver_res0          (data_valid_yed_o-data_valid_yst_o+1'b1),
	.syn_off0_re       (syn_off0_re     ),
	.syn_off0_vs       (syn_off0_vs     ),
	.syn_off0_hs       (syn_off0_hs     ),
	.syn_clip0_vs      (                ),
	.syn_clip0_hs      (                ),
	.syn_clip0_de      (                ),
	.syn_clip0_data    (                )
);

//==========================================================
//monitor
`ifdef RD_VIDEO_WIDTH_16   
	assign m0_clk     = output_pixel_clk ;
	assign m0_vs_rgb  = syn_off0_vs         ;       
	assign m0_hs_rgb  = syn_off0_hs         ;       
	assign m0_de_rgb  = off0_syn_de         ;       
	assign m0_data0_r = {off0_syn_data[ 4: 0],3'd0};
	assign m0_data0_g = {off0_syn_data[10: 5],2'd0};
	assign m0_data0_b = {off0_syn_data[15:11],3'd0};
	assign m0_data1_r = 8'd0             ;
	assign m0_data1_g = 8'd0             ;
	assign m0_data1_b = 8'd0             ;
`endif 

`ifdef RD_VIDEO_WIDTH_24
	assign m0_clk     = output_pixel_clk    ;
	assign m0_vs_rgb  = syn_off0_vs         ;
	assign m0_hs_rgb  = syn_off0_hs         ;
	assign m0_de_rgb  = off0_syn_de         ;
	assign m0_data0_r = off0_syn_data[ 7: 0];
	assign m0_data0_g = off0_syn_data[15: 8];
	assign m0_data0_b = off0_syn_data[23:16];
	assign m0_data1_r = 8'd0                ;
	assign m0_data1_g = 8'd0                ;
	assign m0_data1_b = 8'd0                ;
`endif 

`ifdef RD_VIDEO_WIDTH_32
	assign m0_clk	  = output_pixel_clk    ;
	assign m0_vs_rgb  = syn_off0_vs         ;
	assign m0_hs_rgb  = syn_off0_hs         ;
	assign m0_de_rgb  = off0_syn_de         ;
	assign m0_data0_r = off0_syn_data[ 7: 0];
	assign m0_data0_g = off0_syn_data[15: 8];
	assign m0_data0_b = off0_syn_data[23:16];
	assign m0_data1_r = 8'd0	            ;
	assign m0_data1_g = 8'd0	            ;
	assign m0_data1_b = 8'd0	            ;
`endif 
monitor#
(
	.BMP_OUTPUTED_WIDTH  (BMP0_OUTPUTED_WIDTH ),
	.BMP_OUTPUTED_HEIGHT (BMP0_OUTPUTED_HEIGHT),
	.BMP_OUTPUTED_NAME   (BMP0_OUTPUTED_NAME  ),
	.BMP_OUTPUTED_NUMBER (BMP0_OUTPUTED_NUMBER)
)
monitor_inst0
(
	.link_i       (BMP0_LINK          ), 
	.video2bmp_en (rst_n              ),
	.pixel_clock  (m0_clk             ), 
	.vsync        (m0_vs_rgb          ),        
	.hsync        (m0_hs_rgb          ),        
	.data_valid   (m0_de_rgb          ), 
	.data0_r      (m0_data0_r         ),    
	.data0_g      (m0_data0_g         ),
	.data0_b      (m0_data0_b         ),
	.data1_r      (m0_data1_r         ),    
	.data1_g      (m0_data1_g         ),
	.data1_b      (m0_data1_b         )
);
          
endmodule
